#!/usr/bin/env python3

import argparse
import re
import subprocess
import sys
import traceback


def main():
    args = parse_args()
    update_snap_overrides(args.snaps, dry_run=args.dry_run)


def parse_args():
    ap = argparse.ArgumentParser()
    ap.add_argument('snaps', metavar='snap', nargs='+',
                    help='One or more snaps which should have snap proxy revisiosn updated')
    ap.add_argument('-d', '--dry-run', action='store_true', default=False,
                    help="Don't actually do anything; just print the command lines which should be run.")
    return ap.parse_args()


def update_snap_overrides(snaps, dry_run=False):
    if dry_run:
        print('Please run the following commands to update the snap pinnings:')
    commands = []
    for snap in snaps:
        commands.extend(get_snap_override_commands(snap))
    for command in commands:
        str_cmd = ' '.join(command)
        if dry_run:
            print(str_cmd)
        if not dry_run:
            print(f'Running: {str_cmd}')
            try:
                subprocess.check_call(command)
            except Exception:
                print(traceback.format_exc(), file=sys.stderr)


def get_snap_override_commands(snap):
    commands = []
    channel_rev_pairs = get_channels_and_revisions(snap)
    for channel, rev in channel_rev_pairs:
        # "latest" appears to be a special case and not a normal pinnable channel per
        # snap-proxy; we need to drop the "latest/" prefix to pin those channels.
        if channel.startswith('latest/'):
            channel = channel.split('/', 1)[1]
        commands.append(['snap-proxy', 'override', snap, f'{channel}={rev}'])
    return commands


def get_channels_and_revisions(snap):
    stdout = subprocess.check_output(['snap', 'info', snap, '--unicode=never'])
    parsing_channels = False
    channel_line_regex = re.compile('^  ([^:]+): (.*)$')
    revision_regex = re.compile('^[^(]+\(([^)]+)\).*$')
    channel_rev_pairs = []
    last_revision = None
    for line in stdout.decode(errors='replace').splitlines():
        if not parsing_channels:
            parsing_channels = line.startswith('channels:')
        else:
            if not line.startswith('  '):
                # We've reached the end of the channels section; don't parse any further
                break
            channel_match = channel_line_regex.match(line)
            if not channel_match:
                print(f'WARNING: Unexpected channel line for snap {snap}:\n  {line}',
                      file=sys.stderr)
                continue
            channel, remainder = channel_match.groups()
            remainder = remainder.strip()
            revision_match = revision_regex.match(remainder)
            if revision_match:
                revision = revision_match.group(1)
                channel_rev_pairs.append((channel, revision))
                last_revision = revision
            elif remainder == '^':
                # Seems to imply a pass-through to the next more stable version.
                # It appears we can't actually pin channels which have this, so
                # just skip.
                #channel_rev_pairs.append((channel, last_revision))
                pass
            elif remainder == '--':
                # I think this means "no snap available for this channel".
                # Not exactly sure of the nuance of this vs. "^".
                # Ignore.
                pass
            else:
                print(f'WARNING: Unable to parse channels line for snap {snap}:\n  {line}', file=sys.stderr)
    return channel_rev_pairs


if __name__ == '__main__':
    main()
